@@ -1,11 +1,14 @@
-from .mixins import WeightRegularizerMixin
-from .base_metric_loss_function import BaseMetricLossFunction
-from ..utils import loss_and_miner_utils as lmu, common_functions as c_f
+import math
+
+import numpy as np
 import scipy.special
 import torch
-import math
-import numpy as np
+
 from ..distances import CosineSimilarity
+from ..utils import common_functions as c_f
+from ..utils import loss_and_miner_utils as lmu
+from .base_metric_loss_function import BaseMetricLossFunction
+from .mixins import WeightRegularizerMixin
 
 
 class LargeMarginSoftmaxLoss(WeightRegularizerMixin, BaseMetricLossFunction):
@@ -42,9 +45,9 @@ class LargeMarginSoftmaxLoss(WeightRegularizerMixin, BaseMetricLossFunction):
     def get_cos_with_margin(self, cosine):
         cosine = cosine.unsqueeze(1)
         for attr in ["n_range", "margin_choose_n", "cos_powers", "alternating"]:
-            setattr(self, attr, getattr(self, attr).to(cosine.device))
-        cos_powered = cosine ** self.cos_powers
-        sin_powered = (1 - cosine ** 2) ** self.n_range
+            setattr(self, attr, c_f.to_device(getattr(self, attr), cosine))
+        cos_powered = cosine**self.cos_powers
+        sin_powered = (1 - cosine**2) ** self.n_range
         terms = (
             self.alternating * self.margin_choose_n * cos_powered * sin_powered
         )  # Equation 7 in the paper
@@ -62,14 +65,16 @@ class LargeMarginSoftmaxLoss(WeightRegularizerMixin, BaseMetricLossFunction):
 
     def get_target_mask(self, embeddings, labels):
         batch_size = labels.size(0)
-        mask = torch.zeros(batch_size, self.num_classes, dtype=embeddings.dtype).to(
-            embeddings.device
+        mask = torch.zeros(
+            batch_size,
+            self.num_classes,
+            dtype=embeddings.dtype,
+            device=embeddings.device,
         )
         mask[torch.arange(batch_size), labels] = 1
         return mask
 
-    def modify_cosine_of_target_classes(self, cosine_of_target_classes, *args):
-        _, _, labels, _ = args
+    def modify_cosine_of_target_classes(self, cosine_of_target_classes):
         cos_with_margin = self.get_cos_with_margin(cosine_of_target_classes)
         angles = self.get_angles(cosine_of_target_classes)
         with torch.no_grad():
@@ -85,13 +90,17 @@ class LargeMarginSoftmaxLoss(WeightRegularizerMixin, BaseMetricLossFunction):
         return logits * product_of_magnitudes * self.scale
 
     def cast_types(self, dtype, device):
-        self.W.data = self.W.data.to(device).type(dtype)
-        self.n_range = self.n_range.to(device).type(dtype)
-        self.margin_choose_n = self.margin_choose_n.to(device).type(dtype)
-        self.cos_powers = self.cos_powers.to(device).type(dtype)
-        self.alternating = self.alternating.to(device).type(dtype)
+        self.W.data = c_f.to_device(self.W.data, device=device, dtype=dtype)
+        self.n_range = c_f.to_device(self.n_range, device=device, dtype=dtype)
+        self.margin_choose_n = c_f.to_device(
+            self.margin_choose_n, device=device, dtype=dtype
+        )
+        self.cos_powers = c_f.to_device(self.cos_powers, device=device, dtype=dtype)
+        self.alternating = c_f.to_device(self.alternating, device=device, dtype=dtype)
 
-    def compute_loss(self, embeddings, labels, indices_tuple):
+    def compute_loss(self, embeddings, labels, indices_tuple, ref_emb, ref_labels):
+        c_f.labels_required(labels)
+        c_f.ref_not_supported(embeddings, labels, ref_emb, ref_labels)
         dtype, device = embeddings.dtype, embeddings.device
         self.cast_types(dtype, device)
         miner_weights = lmu.convert_to_weights(indices_tuple, labels, dtype=dtype)
@@ -99,7 +108,7 @@ class LargeMarginSoftmaxLoss(WeightRegularizerMixin, BaseMetricLossFunction):
         cosine = self.get_cosine(embeddings)
         cosine_of_target_classes = cosine[mask == 1]
         modified_cosine_of_target_classes = self.modify_cosine_of_target_classes(
-            cosine_of_target_classes, cosine, embeddings, labels, mask
+            cosine_of_target_classes
         )
         diff = (modified_cosine_of_target_classes - cosine_of_target_classes).unsqueeze(
             1
@@ -120,3 +129,7 @@ class LargeMarginSoftmaxLoss(WeightRegularizerMixin, BaseMetricLossFunction):
 
     def get_default_distance(self):
         return CosineSimilarity()
+
+    def get_logits(self, embeddings):
+        logits = self.get_cosine(embeddings)
+        return self.scale_logits(logits, embeddings)
