@@ -1,8 +1,8 @@
-from .metric_loss_only import MetricLossOnly
-import logging
-from ..utils import common_functions as c_f
 import torch
 
+from ..utils import common_functions as c_f
+from .metric_loss_only import MetricLossOnly
+
 
 class UnsupervisedEmbeddingsUsingAugmentations(MetricLossOnly):
     def __init__(self, transforms, data_and_label_setter=None, *args, **kwargs):
@@ -12,7 +12,7 @@ class UnsupervisedEmbeddingsUsingAugmentations(MetricLossOnly):
         self.transforms = transforms
         self.collate_fn = self.custom_collate_fn
         self.initialize_dataloader()
-        logging.info("Transforms: %s" % transforms)
+        c_f.LOGGER.info("Transforms: %s" % transforms)
 
     def initialize_data_and_label_setter(self):
         if self.data_and_label_setter is None:
